package gui;

import java.awt.AWTException;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.KeyEventDispatcher;
import java.awt.KeyboardFocusManager;
import java.awt.Robot;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.TitledBorder;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;

import receiver.DumpReceiver;
import translator.ParserLib;
import translator.TranslationRule;
import util.FileChooser;
import data.TestRecord;

/**
 * MIDI Qwertyfier class that 
 */
public class MIDIQwertyfier extends JFrame{

	private JLabel ruleLabel = new JLabel("Translation Rule : ");
	private JLabel fileLabel = new JLabel("Not specified");
	private JLabel deviceLabel = new JLabel("Select Input Device : ");
	private JLabel actionLabel = new JLabel("Action : ");
	private JLabel soundLabel = new JLabel("Sound : ");

	private JTextField hiddenField = new JTextField();
	private JTextField actionField = new JTextField();
	private JTextField soundField = new JTextField();
	private JTextArea promptArea = new JTextArea();

	private JMenuBar bar = new JMenuBar();

	private JMenu file = new JMenu("File");
	private JMenuItem create = new JMenuItem("Create new rule");
	private JMenuItem load = new JMenuItem("Load");
	private JMenuItem save = new JMenuItem("Save");
	private JMenuItem export = new JMenuItem("Export Record");
	private JMenuItem exit = new JMenuItem("Exit");

	private JMenu edit = new JMenu("Edit");
	private JMenuItem editRules = new JMenuItem("Edit translation rules");

	private JButton clearButton = new JButton(" Clear ");

	private JComboBox deviceComboBox = new JComboBox();;
	private JScrollPane promptScrollPane = new JScrollPane();

	private DumpReceiver r =  new DumpReceiver();
	private File xmlFile;
	private Robot robot;
	private TestRecord record;
	private long previousTime = -1;

	private static TranslationRule 	rules;
	private static SoundSimulator 	simulator;
	private final MIDIQwertyfier 	me = this;

	static KeyboardFocusManager focusManager;

	/**
	 * Main method for MIDIQwertyfier class
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		}
		catch(ClassNotFoundException e) {
			e.printStackTrace();
		}
		catch(InstantiationException e) {
			e.printStackTrace();
		}
		catch(IllegalAccessException e) {
			e.printStackTrace();
		}
		catch(UnsupportedLookAndFeelException e) {
			e.printStackTrace();
		}

		MIDIQwertyfier app = new MIDIQwertyfier();
		app.setVisible(true);
	}

	/**
	 * MIDIQwertyfier constructor.  Initializes all of its components and
	 * allocate components on the right positions.  Initially it is non-visible.
	 */
	public MIDIQwertyfier(){
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setTitle("MIDI Translation Application");
		setSize(335, 380);
		setLocation(200, 150);
		setResizable(false);

		createAppMenuBar();
		initDeviceBox();

		MIDIQwertyfier.focusManager = KeyboardFocusManager.getCurrentKeyboardFocusManager();
		focusManager.addKeyEventDispatcher(new KeyboardButtonListener());
		try {
			robot = new Robot();
		} catch (AWTException e1) {
			e1.printStackTrace();
		}

		clearButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				promptArea.setText("");
			}
		});

		editRules.setEnabled(false);

		promptArea.setRows(8);
		promptArea.setEditable(true);
		promptScrollPane.setViewportView(promptArea);

		allocateComponents(getContentPane());

		pack();
		
		record = new TestRecord("TEST");
	}

	private void allocateComponents(Container contentPane) {

		JPanel pane = new JPanel();
		pane.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;

		c.insets  = new Insets(5,10,5,10);
		c.gridx = 0;
		c.gridy = 0;
		pane.add(ruleLabel, c);

		c.insets  = new Insets(5,5,5,10);
		c.gridx = 1;
		pane.add(fileLabel, c);

		c.insets  = new Insets(5,10,5,10);
		c.gridx = 0;
		c.gridy = 1;
		pane.add(deviceLabel, c);

		c.insets  = new Insets(5,5,5,10);
		c.gridx = 1;
		pane.add(deviceComboBox, c);

		c.insets  = new Insets(5,10,5,10);
		c.gridx = 0;
		c.gridy = 2;
		pane.add(soundLabel, c);

		c.insets  = new Insets(5,5,5,10);
		c.gridx = 1;
		pane.add(actionLabel, c);

		c.insets  = new Insets(5,10,5,10);
		c.gridx = 0;
		c.gridy = 3;

		pane.add(soundField, c);

		c.insets  = new Insets(5,5,5,20);
		c.gridx = 1;
		pane.add(actionField, c);

		c.insets = new Insets(5, 20, 10, 20);
		c.gridx = 0;
		c.gridy = 4;
		c.gridwidth= 2;
		pane.add(getPromptPane(), c);

		contentPane.add(pane);
	}

	private JPanel getPromptPane(){
		JPanel pane = new JPanel();

		BorderLayout borderLayout = new BorderLayout();
		pane.setLayout(borderLayout);
		borderLayout.setVgap(4);
		pane.setLayout(borderLayout);
		pane.add(promptScrollPane, BorderLayout.PAGE_START);
		pane.add(clearButton, BorderLayout.PAGE_END);
		pane.setBorder(
				BorderFactory.createTitledBorder(
						null, 
						" Prompt Area ", 
						TitledBorder.DEFAULT_JUSTIFICATION, 
						TitledBorder.DEFAULT_POSITION, 
						new Font("Dialog", Font.PLAIN, 10), 
						new Color(51, 51, 51)));
		return pane;
	}

	private void initDeviceBox() {
		Vector<String> devices = new Vector<String>();
		try {
			String[] argument = {"-l"};
			devices = receiver.MidiInDump.showDevices(argument);
		} catch (Exception e1) {
			e1.printStackTrace();
		}

		deviceComboBox = new JComboBox(devices);
		deviceComboBox.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(deviceComboBox.getSelectedIndex() == 0){
					if(simulator != null){
						simulator.close();
						simulator = null;
					}
					if(r != null){
						r.close();
						r = null;

					}
				}
				if(deviceComboBox.getSelectedIndex() > 1){
					if(simulator != null){
						simulator.close();
						simulator = null;
					}
					r =
						(DumpReceiver)(receiver.MidiInDump.selectDevice(Integer.parseInt(
								((String)deviceComboBox.getSelectedItem()).substring(0,1))));
					r.setMIDIApp(me);
					//receiver.MidiInDump.run(inputDevice,r);
				}
				else if(deviceComboBox.getSelectedIndex() == 1){
					if(r != null){
						r.close();
						r = null;
					}
					simulator = new SoundSimulator(me);
				}
			}});
	}



	/**
	 * Getter method for retrieving the TranslationRule that MIDIQwertyfier is currently using
	 * @return
	 */
	public TranslationRule getTranslationRule(){
		return MIDIQwertyfier.rules;
	}

	private void createAppMenuBar(){
		file.setMnemonic('F');
		create.setMnemonic('C');
		load.setMnemonic('L');
		save.setMnemonic('S');
		exit.setMnemonic('x');

		file.add(create);
		file.add(load);
		file.add(save);	
		file.add(export);
		file.add(exit);

		edit.setMnemonic('E');
		edit.add(editRules);

		create.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				JPanel dialogPane = new JPanel(new GridBagLayout());
				GridBagConstraints c = new GridBagConstraints();
				c.fill = GridBagConstraints.BOTH;

				c.insets = new Insets(10,10,4,10);
				c.gridwidth = 2;
				JLabel notice = new JLabel("XML file will be stored in 'rules' folder");
				notice.setFont(new Font("Verdana", Font.BOLD, 13));
				dialogPane.add(notice, c);

				c.insets = new Insets(10,10,4,10);
				c.gridy = 1;
				c.gridwidth = 1;
				dialogPane.add(nameLabel, c);

				c.insets = new Insets(10,0,4,10);
				c.gridx = 1;
				dialogPane.add(nameField, c);

				c.insets = new Insets(0,10,4,10);
				c.gridx = 0;
				c.gridy = 2;
				dialogPane.add(authorLabel, c);

				c.insets = new Insets(0,0,4,10);
				c.gridx = 1;
				dialogPane.add(authorField, c);

				c.insets = new Insets(0,10,10,10);
				c.gridx = 0;
				c.gridy = 3;
				dialogPane.add(commentLabel, c);

				c.insets = new Insets(0,0,10,10);
				c.gridx = 1;
				dialogPane.add(commentField, c);

				c.insets = new Insets(0, 50, 10, 50);
				c.gridx = 0;
				c.gridy = 4;
				c.gridwidth = 2;
				dialogPane.add(getButtonPane(), c);

				createFileDialog = new JDialog(me, "Create New Rule");
				createFileDialog.setContentPane(dialogPane);
				createFileDialog.pack();
				createFileDialog.setVisible(true);
			}

			private JPanel getButtonPane(){
				JPanel pane = new JPanel(new GridLayout(1, 2, 30, 20));
				initializeButtons();
				pane.add(createButton);
				pane.add(cancelButton);
				return pane;
			}

			private void initializeButtons(){
				createButton.addActionListener(new CreateButtonListener());
				cancelButton.addActionListener(new CancelButtonListener());
			}

			class CreateButtonListener implements ActionListener {
				public void actionPerformed(ActionEvent evt) {
					if(nameField.getText().length() == 0){
						JOptionPane.showMessageDialog(me, "Please enter file name.",
								"Warning", JOptionPane.WARNING_MESSAGE);
						return;
					}
					if(authorField.getText().length() == 0){
						JOptionPane.showMessageDialog(me, "Please enter name of author.",
								"Warning", JOptionPane.WARNING_MESSAGE);
						return;
					}

					Element rootNode = new Element("MIDITranslation");
					rootNode.setAttribute("Author", authorField.getText());
					rootNode.setAttribute("Comment", commentField.getText());

					Element rulesNode = new Element("TranslationRules");
					rootNode.addContent(rulesNode);

					Document doc = new Document();
					doc.setRootElement(rootNode);


					String fileName = nameField.getText();
					File newRuleFile = new File("./rules/", fileName);
					if(newRuleFile.exists()){
						JOptionPane.showMessageDialog(me, "There is already a file with the same name.  Please rename the file.",
								"Warning", JOptionPane.WARNING_MESSAGE);
						return;
					}
					FileOutputStream fos;
					try {
						fos = new FileOutputStream(newRuleFile);
						newRuleFile.createNewFile();

						MIDIQwertyfier.rules = new TranslationRule(doc);
						xmlFile = newRuleFile;
						fileLabel.setText(xmlFile.getName());
						editRules.setEnabled(true);

						ParserLib.xmlOutputter.output(rules.getDocument(), System.out);
						ParserLib.xmlOutputter.output(rules.getDocument(), fos);
					} catch (IOException ex) {
						ex.printStackTrace();
					}

				}
			}

			class CancelButtonListener implements ActionListener {
				public void actionPerformed(ActionEvent evt) {
					createFileDialog.dispose();
				}
			}
			private JDialog createFileDialog;
			private JLabel nameLabel = new JLabel ("File Name : ");
			private JLabel authorLabel = new JLabel ("Author : ");
			private JLabel commentLabel = new JLabel ("Comment : ");
			private JTextField nameField = new JTextField();
			private JTextField authorField = new JTextField();
			private JTextField commentField = new JTextField();
			private JButton createButton = new JButton("Create");
			private JButton cancelButton = new JButton("Cancel");
		});

		load.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				try {
					xmlFile = FileChooser.choose();
					if(xmlFile != null){
						MIDIQwertyfier.rules = new TranslationRule(xmlFile.getAbsolutePath());
						fileLabel.setText(xmlFile.getName());
						editRules.setEnabled(true);
					}
				} catch (IOException e1) {
					e1.printStackTrace();
					System.err.println("ERROR : No such file found");
				} catch (JDOMException e1) {
					e1.printStackTrace();
					System.err.println("ERROR : Invalid file format");
				}
			}
		});

		save.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				FileOutputStream fos;
				try {
					fos = new FileOutputStream(xmlFile);
					xmlFile.delete();
					xmlFile.createNewFile();
					ParserLib.xmlOutputter.output(MIDIQwertyfier.rules.getDocument(), System.out);
					ParserLib.xmlOutputter.output(MIDIQwertyfier.rules.getDocument(), fos);
				} catch (IOException ex) {
					ex.printStackTrace();
				}
			}
		});
		
		export.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				record.outputToFile();
			}
		});
		
		exit.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				me.dispose();
				System.exit(1);
			}
		});
		
		editRules.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				if(MIDIQwertyfier.rules==null){
					System.err.println("Translation Rule is null");
					return;
				}
				new ConfigureRuleDialog(rules);
			}
		});	
		bar.add(file);
		bar.add(edit);
		setJMenuBar(bar);
	}

	/**
	 * Update the sound text field, action text field and prompt text area using
	 * the Sound that it receives
	 * @param sound
	 */
	public void update(String sound){
		
		System.out.println("Sound is "+sound);
		
		if(sound.equals("")) return;

		String action = "";
		try{
			action = MIDIQwertyfier.rules.getAction(sound);
		}
		catch(NullPointerException e){	}

		//System.out.println("Sound: "+sound+"    Action: "+action);
		long currentTime = System.currentTimeMillis();
		if(previousTime != -1 && !action.equals(""))
			record.addData(action, currentTime - previousTime);
		previousTime = currentTime;

		soundField.setText(sound);
		actionField.setText(action);

		if(action.length() > 0){ 
			if(action.equals("[new line]")){
				robot.keyPress(KeyEvent.VK_ENTER );
				robot.keyRelease(KeyEvent.VK_ENTER);
				return;
			}
			if(action.equals("[backspace]")){
				robot.keyPress(KeyEvent.VK_BACK_SPACE);
				robot.keyRelease(KeyEvent.VK_BACK_SPACE);
				return;
			}
			
			for(int i = 0; i < action.length(); i++) invokeRobotAction(action.charAt(i));
		}
		
		promptArea.setText(promptArea.getText()+action);
	}

	private void invokeRobotAction(char c){
        boolean isShiftKeyPressed = false;
        if(c >= '0' && c <= '9'){
        	//System.out.println("NUMBER: Typing "+c+" ("+Integer.valueOf(c)+")");
        }
        else if(c >= 'a' && c <= 'z'){
        	//System.out.println("NONCAP: Typing "+c+" ("+Integer.valueOf(c)+")");
            c = (char) (c - 32);
        }
        else if(c >= 'A' && c <= 'Z'){
        	//System.out.println("CAPLET: Typing "+c+" ("+Integer.valueOf(c)+")");
            isShiftKeyPressed = true;
        }
        else{
        	//System.out.println("SPECIAL: Typing "+c+" ("+Integer.valueOf(c)+")");
        	switch(c){
        	/*case ',':	c = KeyEvent.VK_COMMA;			break;
        	case '-':	c = KeyEvent.VK_MINUS;			break;
        	case '/':	c = KeyEvent.VK_SLASH;			break;
        	case ';':	c = KeyEvent.VK_SEMICOLON;		break;
        	case '=':	c = KeyEvent.VK_EQUALS;			break;
        	case '[':	c = KeyEvent.VK_OPEN_BRACKET;	break;
        	case ']':	c = KeyEvent.VK_CLOSE_BRACKET;	break;
        	case '*':	c = KeyEvent.VK_ASTERISK;		break;
        	case '"':	c = KeyEvent.VK_QUOTEDBL;		break;
        	case '<':	c = KeyEvent.VK_LESS;			break;
        	case '>':	c = KeyEvent.VK_GREATER;		break;
        	case '{':	c = KeyEvent.VK_BRACELEFT;		break;
        	case '}':	c = KeyEvent.VK_BRACERIGHT;		break;
        	case '@':	c = KeyEvent.VK_AMPERSAND;		break;
        	case ':':	c = KeyEvent.VK_COLON;			break;
        	case '^':	c = KeyEvent.VK_CIRCUMFLEX;		break;
        	case '$':	c = KeyEvent.VK_DOLLAR;			break;
        	case '!':	c = KeyEvent.VK_EXCLAMATION_MARK;	break;
        	case '(':	c = KeyEvent.VK_LEFT_PARENTHESIS;	break;
        	case ')':	c = KeyEvent.VK_RIGHT_PARENTHESIS;	break;
        	case '+':	c = KeyEvent.VK_PLUS;			break;
        	case '#':	c = KeyEvent.VK_NUMBER_SIGN;	break;
        	case '_':	c = KeyEvent.VK_UNDERSCORE;		break;*/

        	case '\'':	c = KeyEvent.VK_QUOTE;
        	case '\\':
        	case '`':
        	case ',':
        	case '-':
        	case '/':
        	case ';':
        	case '=':
        	case '[':
        	case ']':	break;
        	default:
        		isShiftKeyPressed = true;
        		switch(c){
            	case ')':	c = KeyEvent.VK_0;	break;
            	case '!':	c = KeyEvent.VK_1;	break;
            	case '@':	c = KeyEvent.VK_2;	break;
            	case '#':	c = KeyEvent.VK_3;	break;
            	case '$':	c = KeyEvent.VK_4;	break;
            	case '%':	c = KeyEvent.VK_5;	break;
            	case '^':	c = KeyEvent.VK_6;	break;
            	case '&':	c = KeyEvent.VK_7;	break;
            	case '*':	c = KeyEvent.VK_8;	break;
            	case '(':	c = KeyEvent.VK_9;	break;

        		case '?':	c = KeyEvent.VK_SLASH;	break;

        		case '"':	c = KeyEvent.VK_QUOTE;	break;
            	case '<':	c = ',';	break;
            	case '>':	c = '.';	break;
            	case '{':	c = '[';	break;
            	case '}':	c = ']';	break;
            	case ':':	c = ';';	break;
            	case '+':	c = '=';	break;
            	case '_':	c = '-';	break;
            	case '|':	c = '\\';	break;
            	case '~':	c = '`';	break;
        		default:
        			System.out.println("Undefined keyboard mapping for ("+c+")");
        			robot.keyPress(c);
        			robot.keyRelease(c);
        			return;
        		}
        	}
        }
        if(isShiftKeyPressed) robot.keyPress(KeyEvent.VK_SHIFT);
        robot.keyPress(c);
        robot.keyRelease(c);
        if(isShiftKeyPressed) robot.keyRelease(KeyEvent.VK_SHIFT);
    }




	public void setSoundField(String sound){
		soundField.setText(sound);
	}

	private class KeyboardButtonListener implements KeyEventDispatcher{
		public boolean dispatchKeyEvent(KeyEvent e) {
			Object source = e.getSource();
			//System.out.println(e.getKeyChar() + " "+ e.getKeyCode());
			if(source == actionField || source == soundField || source == promptArea){
			//if(source == actionField || source == soundField){
				//focusManager.redispatchEvent(hiddenField, e);
			}
			return false;
		}
	}
}
