package test;


import javax.sound.midi.*;

import org.jdom.JDOMException;

import receiver.DumpReceiver;
import translator.TranslationRule;

//import receiver.MidiMessageReceiver;

import java.io.IOException;


/**
 * This application tests the recognizer and the translator.
 * @author ksun011
 */

public class Test{
	private static final int velocity = 69;
	
	/**
	 * The main application. 
	 * @param args
	 * @throws InvalidMidiDataException
	 * @throws JDOMException
	 * @throws IOException
	 */
	public static void main(String args[])throws InvalidMidiDataException, JDOMException, IOException{
		
		Receiver r = new DumpReceiver(System.out, true);
		TranslationRule rule = null;
			
			try {
				((DumpReceiver)r).rule= new TranslationRule("./TestRules.xml");
				rule = ((DumpReceiver)r).rule;
				System.out.println("Test 1 : XML file loaded successfully");
			} catch (IOException e) {
				e.printStackTrace();
			} catch (JDOMException e) {
				e.printStackTrace();
			}
			
		
		long time = 1000000;
		long duration = 500000;
		
		/**
		 * Testing receiver package with generated MIDIEvents
		 */
		
		// Test receiving a Note C8
		// This test should print [backspace] on the screen
		noteOn(r,time,108,velocity);
		noteOff(r,time+duration,108,velocity);
		
		// Test receiving a chord C4E4G4 
		// This test should print Software Engineering 702 on the screen
		noteOn(r,time,60,velocity);
		noteOn(r,time,64,velocity);
		noteOn(r,time,67,velocity);
		noteOff(r,time+duration,60,velocity);
		noteOff(r,time+duration,64,velocity);
		noteOff(r,time+duration,67,velocity);
		
		
		/**
		 * Testing adding, modifying and removing rules
		 */
		//Testing whether the rule correctly returns action
		System.out.println("Translation test 1: "+rule.getAction("C4E4G4").equals("Software Engineering 702"));
		
		//Testing whether the rule gets modified correctly
		rule.modifyAction("C4E4G4", "Computer Science 705");
		System.out.println("Translation test 2: "+rule.getAction("C4E4G4").equals("Computer Science 705"));
		
		//Testing whether a rule gets removed successfully
		rule.removeRule("C4E4G4");
		System.out.println("Translation test 3: "+rule.getAction("C4E4G4").equals(""));
		
		//Testing whether adding a new rule works properly
		rule.addRule("C4E4G4", "Successfully Modified");
		System.out.println("Translation test 4: "+rule.getAction("C4E4G4").equals("Successfully Modified"));
	}

	
	
	
	
	/**
	 * Simulates a NOTEON Midi event.
	 * @param timestamp - the time of the event
	 * @param key - the key of the note that is being pressed
	 * @param velocity - the velocity of the note that is being pressed
	 */
	public static void noteOn(Receiver r, long timestamp, int key, int velocity)
										throws InvalidMidiDataException{ 
		ShortMessage on = new ShortMessage();
		on.setMessage(ShortMessage.NOTE_ON,  0, key, velocity);
		
		r.send (on, timestamp);
	}
	
	/**
	 * Simulates a NOTEOFF Midi event.
	 * @param timestamp - the time of the event
	 * @param key - the key of the note that is being released
	 * @param velocity - the velocity of the note that is being released
	 */
	public static void  noteOff(Receiver r, long timestamp, int key, int velocity)
										throws InvalidMidiDataException{
		ShortMessage off = new ShortMessage();
		off.setMessage(ShortMessage.NOTE_OFF,  0, key, velocity);
		
		r.send (off, timestamp);
	}
}