package util;

import java.awt.Color;
import java.awt.Font;
import java.util.Map;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import org.jdom.Element;

import translator.TranslationRule;

public class TranslationTable extends JTable{
	public TranslationTable(TranslationRule r){
		super(r==null ? 0:r.getHashtable().size(), 2);
		
		DefaultTableModel tableModel = (DefaultTableModel)getModel();
		tableModel.setDataVector(getRulesVector(r), getColumnHeading(r));
		
		Font smallFont = new Font("Dialog", Font.PLAIN, 10);
		
		setGridColor(Color.gray);
		setFont(smallFont);
		setBorder(
				BorderFactory.createCompoundBorder(
						BorderFactory.createLineBorder(Color.black), 
						BorderFactory.createEmptyBorder(2, 4, 2, 4)));
	}
	
	/**
	 * Gets rule table column headings
	 * @param r 
	 * @return Vector<String> Sound Input, Action
	 */
	private Vector<String> getColumnHeading(TranslationRule r){
		Vector<String> columnHeading = new Vector<String>();
		columnHeading.add("Action");
		columnHeading.add("Sound Input");
		return columnHeading;
	}

	/**
	 * Get Vector containing Vectors defining each rule
	 * @param r 
	 * @return Vector<Vector> Each Vector contains String Sound (Key) and String Action
	 */
	private Vector<Vector<String>> getRulesVector(TranslationRule r){
		Vector<Vector<String>> ruleVector = new Vector<Vector<String>>();

		try{
			Vector<String> singleRule;

			for(Map.Entry<String, Element> m : r.getHashtable().entrySet()){
				singleRule = new Vector<String>();
				singleRule.add(m.getValue().getAttributeValue("Action"));
				singleRule.add(m.getKey());
				ruleVector.add(singleRule);
			}
		}
		catch(NullPointerException e){}

		//sort
		int pointer = 0;
		Vector<String> v;
		for(int i = 0; i < ruleVector.size(); i++){
			for(int j = i+1; j < ruleVector.size(); j++){
				if(ruleVector.get(i).get(0).compareTo(ruleVector.get(j).get(0)) > 0){
					v = ruleVector.get(i);
					ruleVector.set(i, ruleVector.get(j));
					ruleVector.set(j, v);
				}
			}
		}
		
		return ruleVector;
	}
}
